@@ -58,6 +58,10 @@ class CliCallback:
         """Not used"""
         pass
 
+    def update_item_size(self, op, opid, size):
+        """Not used"""
+        pass
+
     def worker_done(self, worker, really_delete):
         """Not used"""
         pass
@@ -163,7 +167,6 @@ def process_cmd_line():
     if options.debug_log:
         import logging
         logger = logging.getLogger(__name__)
-        logger.setLevel(logging.DEBUG)
         logger.addHandler(logging.FileHandler(options.debug_log))
         logger.info('BleachBit version %s' % APP_VERSION)
         logger.info(Diagnostic.diagnostic_info())
