@@ -178,6 +178,12 @@ class PreferencesDialog:
             'toggled', self.__toggle_callback, 'delete_confirmation')
         vbox.pack_start(cb_popup, False)
 
+        # Use base 1000 over 1024?
+        cb_units_iec = gtk.CheckButton(
+            _("Use IEC sizes (1 KiB = 1024 bytes) instead of SI (1 kB = 1000 bytes)"))
+        cb_units_iec.set_active(options.get("units_iec"))
+        cb_units_iec.connect('toggled', self.__toggle_callback, 'units_iec')
+        vbox.pack_start(cb_units_iec, False)
         return vbox
 
     def __drives_page(self):
@@ -261,7 +267,7 @@ class PreferencesDialog:
 
         notice = gtk.Label(
             _("All languages will be deleted except those checked."))
-        vbox.pack_start(notice)
+        vbox.pack_start(notice, False)
 
         # populate data
         liststore = gtk.ListStore('gboolean', str, str)
@@ -293,7 +299,7 @@ class PreferencesDialog:
         swindow.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
         swindow.set_size_request(300, 200)
         swindow.add(treeview)
-        vbox.pack_start(swindow, False)
+        vbox.pack_start(swindow)
         return vbox
 
     def __locations_page(self, page_type):
@@ -409,7 +415,7 @@ class PreferencesDialog:
         elif LOCATIONS_CUSTOM == page_type:
             notice = gtk.Label(
                 _("These locations can be selected for deletion."))
-        vbox.pack_start(notice)
+        vbox.pack_start(notice, False)
 
         # create treeview
         treeview = gtk.TreeView(liststore)
@@ -431,7 +437,7 @@ class PreferencesDialog:
         swindow.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
         swindow.set_size_request(300, 200)
         swindow.add(treeview)
-        vbox.pack_start(swindow, False)
+        vbox.pack_start(swindow)
 
         # buttons that modify the list
         button_add_file = gtk.Button(_p('button', 'Add file'))
