@@ -39,12 +39,14 @@ These are the terms:
 
 
 import glob
+import logging
 import os
 import platform
 import re
 import sys
 import traceback
 
+from decimal import Decimal
 
 if 'win32' == sys.platform:
     import _winreg
@@ -76,10 +78,11 @@ def browse_file(_, title):
                                         | win32con.OFN_HIDEREADONLY,
                                         Title=title)
     except pywintypes.error, e:
+        logger = logging.getLogger(__name__)
         if 0 == e.winerror:
-            print 'debug: browse_file(): user cancelled'
+            logger.debug('browse_file(): user cancelled')
         else:
-            traceback.print_exc()
+            logger.exception('exception in browse_file()')
         return None
     return ret[0]
 
@@ -94,10 +97,11 @@ def browse_files(_, title):
                                         | win32con.OFN_HIDEREADONLY,
                                         Title=title)
     except pywintypes.error, e:
+        logger = logging.getLogger(__name__)
         if 0 == e.winerror:
-            print 'debug: browse_files(): user cancelled'
+            logger.debug('browse_files(): user cancelled')
         else:
-            traceback.print_exc()
+            logger.exception('exception in browse_files()')
         return None
     _split = ret[0].split('\x00')
     if 1 == len(_split):
@@ -122,21 +126,30 @@ def browse_folder(hwnd, title):
 
 def csidl_to_environ(varname, csidl):
     """Define an environment variable from a CSIDL for use in CleanerML and Winapp2.ini"""
+    logger = logging.getLogger(__name__)
     try:
         sppath = shell.SHGetSpecialFolderPath(None, csidl)
-        set_environ(varname, sppath)
     except:
-        import traceback
-        traceback.print_exc()
-        print 'ERROR: setting environment variable "%s": %s ' % (
-            varname, str(sys.exc_info()[1]))
+        logger.exception(
+            'exception when getting special folder path for %s' % varname)
+        return
+    # there is exception handling in set_environ()
+    set_environ(varname, sppath)
 
 
 def delete_locked_file(pathname):
     """Delete a file that is currently in use"""
     if os.path.exists(pathname):
-        win32api.MoveFileEx(
-            pathname, None, win32con.MOVEFILE_DELAY_UNTIL_REBOOT)
+        try:
+            win32api.MoveFileEx(
+                pathname, None, win32con.MOVEFILE_DELAY_UNTIL_REBOOT)
+        except WindowsError, e:
+            if not 5 == e.winerror:
+                raise e
+            # show more useful message than "error: (5, 'MoveFileEx', 'Access
+            # is denied.')"
+            raise RuntimeError(
+                'Access denied when attempting to delete locked file %s' % pathname)
 
 
 def delete_registry_value(key, value_name, really_delete):
@@ -252,18 +265,26 @@ def elevate_privileges():
     privileges.  If successful, return True (so original process
     can exit).  If failed or not applicable, return False."""
 
-    if platform.version() <= '5.2':
-        # Earlier than Vista
+    if parse_windows_build() < 6:
+        # Windows XP does not have the UAC.
+        # Vista is the first version Windows that has the UAC.
+        # 5.1 = Windows XP
+        # 6.0 = Vista
+        # 6.1 = 7
+        # 6.2 = 8
+        # 10 = 10
         return False
 
+    logger = logging.getLogger(__name__)
+
     if shell.IsUserAnAdmin():
-        print 'debug: already an admin (UAC not required)'
+        logger.debug('already an admin (UAC not required)')
         return False
 
     if hasattr(sys, 'frozen'):
         # running frozen in py2exe
         exe = unicode(sys.executable, sys.getfilesystemencoding())
-        py = "--gui --no-uac"
+        parameters = "--gui --no-uac"
     else:
         # __file__ is absolute path to bleachbit/Windows.py
         pydir = os.path.dirname(unicode(__file__, sys.getfilesystemencoding()))
@@ -272,29 +293,31 @@ def elevate_privileges():
         # the administrator may not have privileges and user will not be
         # prompted.
         if len(pyfile) > 0 and path_on_network(pyfile):
-            print "debug: skipping UAC because '%s' is on network" % pyfile
+            logger.debug(
+                "debug: skipping UAC because '%s' is on network" % pyfile)
             return False
-        py = '"%s" --gui --no-uac' % pyfile
+        parameters = '"%s" --gui --no-uac' % pyfile
         exe = sys.executable
 
     # add any command line parameters such as --debug-log
-    py = "%s %s" % (py, ' '.join(sys.argv[1:]))
+    parameters = "%s %s" % (parameters, ' '.join(sys.argv[1:]))
 
-    print 'debug: exe=', exe, ' parameters=', py
+    logger.debug('elevate_privileges() exe=%s, parameters=%s' %
+                 (exe, parameters))
 
     rc = None
     try:
         rc = shell.ShellExecuteEx(lpVerb='runas',
                                   lpFile=exe,
-                                  lpParameters=py,
+                                  lpParameters=parameters,
                                   nShow=win32con.SW_SHOW)
     except pywintypes.error, e:
         if 1223 == e.winerror:
-            print 'debug: user denied the UAC dialog'
+            logger.debug('user denied the UAC dialog')
             return False
         raise
 
-    print 'debug: ShellExecuteEx=', rc
+    logger.debug('ShellExecuteEx=%s' % rc)
 
     if isinstance(rc, dict):
         return True
@@ -327,7 +350,8 @@ def get_autostart_path():
     except:
         # example of failure
         # http://bleachbit.sourceforge.net/forum/error-windows-7-x64-bleachbit-091
-        traceback.print_exc()
+        logger = logging.getLogger(__name__)
+        logger.exception('exception in get_autostart_path()')
         msg = 'Error finding user startup folder: %s ' % (
             str(sys.exc_info()[1]))
         import GuiBasic
@@ -349,7 +373,12 @@ def get_fixed_drives():
     """Yield each fixed drive"""
     for drive in win32api.GetLogicalDriveStrings().split('\x00'):
         if win32file.GetDriveType(drive) == win32file.DRIVE_FIXED:
-            yield drive
+            # Microsoft Office 2010 Starter creates a virtual drive that
+            # looks much like a fixed disk but isdir() returns false
+            # and free_space() returns access denied.
+            # https://bugs.launchpad.net/bleachbit/+bug/1474848
+            if os.path.isdir(drive):
+                yield drive
 
 
 def get_known_folder_path(folder_name):
@@ -357,8 +386,9 @@ def get_known_folder_path(folder_name):
 
     Requires Windows Vista, Server 2008, or later
 
-    Based on the code Michael Kropat (mkropat) from <https://gist.github.com/mkropat/7550097> licensed
-    under the GNU GPL"""
+    Based on the code Michael Kropat (mkropat) from
+    <https://gist.github.com/mkropat/7550097>
+    licensed  under the GNU GPL"""
     import ctypes
     from ctypes import wintypes
     from uuid import UUID
@@ -407,7 +437,7 @@ def get_known_folder_path(folder_name):
     pPath = ctypes.c_wchar_p()
     S_OK = 0
     if _SHGetKnownFolderPath(ctypes.byref(fid), 0, UserHandle.current, ctypes.byref(pPath)) != S_OK:
-        raise PathNotFoundException()
+        raise PathNotFoundException(folder_name)
     path = pPath.value
     _CoTaskMemFree(pPath)
     return path
@@ -428,6 +458,13 @@ def get_recycle_bin():
             yield path
 
 
+def get_windows_version():
+    """Get the Windows major and minor version in a decimal like 10.0"""
+    v = win32api.GetVersionEx(0)
+    vstr = '%d.%d' % (v[0], v[1])
+    return Decimal(vstr)
+
+
 def is_process_running(name):
     """Return boolean whether process (like firefox.exe) is running"""
 
@@ -478,7 +515,8 @@ def is_process_running_win32(name):
             if len(clean_modname) > 0 and '?' != clean_modname:
                 # Filter out non-ASCII characters which we don't need
                 # and which may cause display warnings
-                clean_modname2 = re.sub('[^a-z\.]', '_', clean_modname.lower())
+                clean_modname2 = re.sub(
+                    r'[^a-z\.]', '_', clean_modname.lower())
                 if clean_modname2 == name.lower():
                     return True
 
@@ -491,7 +529,7 @@ def is_process_running_wmic(name):
     Works on Windows XP Professional but not on XP Home
     """
 
-    clean_name = re.sub('[^A-Za-z\.]', '_', name).lower()
+    clean_name = re.sub(r'[^A-Za-z\.]', '_', name).lower()
     args = ['wmic', 'path', 'win32_process', 'where', "caption='%s'" %
             clean_name, 'get', 'Caption']
     (_, stdout, _) = General.run_external(args)
@@ -500,11 +538,21 @@ def is_process_running_wmic(name):
 
 def move_to_recycle_bin(path):
     """Move 'path' into recycle bin"""
-    from win32com.shell import shell, shellcon
     shell.SHFileOperation(
         (0, shellcon.FO_DELETE, path, None, shellcon.FOF_ALLOWUNDO | shellcon.FOF_NOCONFIRMATION))
 
 
+def parse_windows_build(build=None):
+    """
+    Parse build string like 1.2.3 or 1.2 to numeric,
+    ignoring the third part, if present.
+    """
+    if not build:
+        # If not given, default to current system's version
+        return get_windows_version()
+    return Decimal('.'.join(build.split('.')[0:2]))
+
+
 def path_on_network(path):
     """Check whether 'path' is on a network drive"""
     if len(os.path.splitunc(path)[0]) > 0:
@@ -524,10 +572,15 @@ def shell_change_notify():
 
 def set_environ(varname, path):
     """Define an environment variable for use in CleanerML and Winapp2.ini"""
+    logger = logging.getLogger(__name__)
     if not path:
+        logger.debug('set_environ(%s, %s): skipping because blank path' %
+                     (varname, path))
         # Such as LocalAppDataLow on XP
         return
     if os.environ.has_key(varname):
+        logger.debug('set_environ(%s, %s): skipping because environment variable is already defined' %
+                     (varname, path))
         # Do not redefine the environment variable when it already exists
         return
     try:
@@ -535,11 +588,10 @@ def set_environ(varname, path):
             raise RuntimeError(
                 'Variable %s points to a non-existent path %s' % (varname, path))
         os.environ[varname] = path
+        logger.debug('set_environ(%s, %s), set' % (varname, path))
     except:
-        import traceback
-        traceback.print_exc()
-        print 'ERROR: setting environment variable "%s": %s ' % (
-            varname, str(sys.exc_info()[1]))
+        logger.exception(
+            'set_environ(%s, %s): exception when setting environment variable' % (varname, path))
 
 
 def setup_environment():
@@ -554,14 +606,19 @@ def setup_environment():
     # LocalLowAppData does not have a CSIDL for use with
     # SHGetSpecialFolderPath. Instead, it is identified using
     # SHGetKnownFolderPath in Windows Vista and later
-    path = get_known_folder_path('LocalAppDataLow')
-    set_environ('LocalAppDataLow', path)
+    try:
+        path = get_known_folder_path('LocalAppDataLow')
+    except:
+        logger = logging.getLogger(__name__)
+        logger.exception('exception identifying LocalAppDataLow')
+    else:
+        set_environ('LocalAppDataLow', path)
 
 
 def split_registry_key(full_key):
-    """Given a key like HKLM\Software split into tuple (hive, key).
+    r"""Given a key like HKLM\Software split into tuple (hive, key).
     Used internally."""
-    assert (len(full_key) >= 6)
+    assert len(full_key) >= 6
     [k1, k2] = full_key.split("\\", 1)
     hive_map = {
         'HKCR': _winreg.HKEY_CLASSES_ROOT,
